@@ -1,6 +1,5 @@
 #!/usr/bin/env python
-#
-# Copyright 2010 Google Inc.
+# Copyright 2010 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -239,6 +238,9 @@ class OutputWriter(json_util.JsonMixin):
 
     Args:
       mapper_spec: instance of model.MapperSpec.
+
+    Returns:
+      boolean. Whether this output writer instance supports slice recovery.
     """
     return False
 
@@ -345,6 +347,9 @@ class _FilePool(context.Pool):
     Args:
       filename: the name of the file as string.
       data: data as string.
+
+    Raises:
+      Error: If it can't append the data to the file.
     """
     if self._size + len(data) > self._flush_size:
       self.flush()
@@ -528,6 +533,12 @@ class FileOutputWriterBase(OutputWriter):
     Args:
       mapreduce_state: mapreduce state as model.MapreduceState.
       mapper_spec: mapper specification as model.MapperSpec
+
+    Returns:
+      The output sharding parameter value.
+
+    Raises:
+      Error: If neither of the two parameters are provided.
     """
     if mapper_spec:
       return _get_params(mapper_spec).get(
@@ -544,6 +555,9 @@ class FileOutputWriterBase(OutputWriter):
 
     Args:
       mapper_spec: an instance of model.MapperSpec to validate.
+
+    Raises:
+      BadWriterParamsError: if the specification is invalid for any reason.
     """
     if mapper_spec.output_writer_class() != cls:
       raise errors.BadWriterParamsError("Output writer class mismatch")
@@ -561,7 +575,7 @@ class FileOutputWriterBase(OutputWriter):
           "Filesystem '%s' is not supported. Should be one of %s" %
           (filesystem, files.FILESYSTEMS))
     if filesystem == files.GS_FILESYSTEM:
-      if not cls.GS_BUCKET_NAME_PARAM in params:
+      if cls.GS_BUCKET_NAME_PARAM not in params:
         raise errors.BadWriterParamsError(
             "%s is required for Google store filesystem" %
             cls.GS_BUCKET_NAME_PARAM)
@@ -648,7 +662,6 @@ class FileOutputWriterBase(OutputWriter):
     """
     return cls(state["filename"], state["request_filename"])
 
-
   def to_json(self):
     """Returns writer state to serialize in json.
 
@@ -662,6 +675,12 @@ class FileOutputWriterBase(OutputWriter):
     """Inherit doc.
 
     Only shard with output per shard can be retried.
+
+    Args:
+      tstate: the transient shard state.
+
+    Returns:
+      True or false if this transient shard state supports sharding retries.
     """
     output_sharding = self._get_output_sharding(
         mapper_spec=tstate.mapreduce_spec.mapper)
@@ -772,6 +791,9 @@ class FileRecordsOutputWriter(FileOutputWriterBase):
 
     Args:
       mapper_spec: an instance of model.MapperSpec to validate.
+
+    Raises:
+      BadWriterParamsError: if the specification is invalid for any reason.
     """
     if cls.OUTPUT_SHARDING_PARAM in _get_params(mapper_spec):
       raise errors.BadWriterParamsError(
@@ -936,7 +958,7 @@ class _GoogleCloudStorageOutputWriter(OutputWriter):
       a string containing the filename.
 
     Raises:
-      BadWriterParamsError if the template contains any errors such as invalid
+      BadWriterParamsError: if the template contains any errors such as invalid
         syntax or contains unknown substitution placeholders.
     """
     naming_format = cls._TMP_FILE_NAMING_FORMAT
@@ -967,7 +989,7 @@ class _GoogleCloudStorageOutputWriter(OutputWriter):
       mapper_spec: an instance of model.MapperSpec.
 
     Raises:
-      BadWriterParamsError if the specification is invalid for any reason such
+      BadWriterParamsError: if the specification is invalid for any reason such
         as missing the bucket name or providing an invalid bucket name.
     """
     writer_spec = _get_params(mapper_spec, allow_old=False)
@@ -1007,6 +1029,7 @@ class _GoogleCloudStorageOutputWriter(OutputWriter):
 
   @classmethod
   def _create(cls, writer_spec, filename_suffix):
+    """Helper method that actually creates the file in cloud storage."""
     # GoogleCloudStorage format for filenames, Initial slash is required
     filename = "/%s/%s" % (writer_spec[cls.BUCKET_NAME_PARAM], filename_suffix)
 
@@ -1048,6 +1071,9 @@ class _GoogleCloudStorageOutputWriter(OutputWriter):
     return writer
 
   def to_json(self):
+    # TODO(user): Move all IO logic to end_slice in new API.
+    if not self._streaming_buffer.closed:
+      self._streaming_buffer.flush()
     result = {self._JSON_GCS_BUFFER: pickle.dumps(self._streaming_buffer),
               self._JSON_NO_DUP: self._no_dup}
     if self._no_dup:
@@ -1078,8 +1104,6 @@ class _GoogleCloudStorageOutputWriter(OutputWriter):
     self._streaming_buffer.close()
 
     if self._no_dup:
-      # TODO(user): This doesn't work properly when the filenames have
-      # spaces in them. It's not being re-quoted properly. b/12066572
       cloudstorage_api._copy2(
           self._streaming_buffer.name,
           self._streaming_buffer.name,
