@@ -31,8 +31,8 @@ import logging
 import os
 import time
 
-from mapreduce.lib import pipeline
-from mapreduce.lib.pipeline import common as pipeline_common
+from mapreduce.third_party import pipeline
+from mapreduce.third_party.pipeline import common as pipeline_common
 from google.appengine.api import files
 from google.appengine.api import modules
 from google.appengine.api.files import file_service_pb
